#region Copyright
//
// (C) Copyright 2011 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted,
// provided that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject 
// to restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Copyright

#region Namespaces
using System;
using System.Collections.Generic;
using System.Text;

using Autodesk.Revit;
using Autodesk.Revit.DB;
using Autodesk.Revit.UI;
using Autodesk.Revit.Attributes;
using Autodesk.Revit.ApplicationServices;
#endregion

namespace ConstructionModelingSample
{
  /// <summary>
  /// VKzfOAPI̎gp@fXg[V 
  /// </summary>
  [Transaction(TransactionMode.Manual)]
  public class CreateAssemblies : IExternalCommand
  {
    #region IExternalCommand Members

    public Result Execute(
      ExternalCommandData commandData,
      ref string message, 
      ElementSet elements
      )
    {
         
      // R}hf[^AvP[VƃhLg擾

      UIApplication uiapp = commandData.Application;
      UIDocument uidoc = uiapp.ActiveUIDocument;
      Document doc = uidoc.Document;

      ICollection<ElementId> elemIds = 
        uidoc.Selection.GetElementIds();
      ElementId categoryId = null;
      ElementId keyElemId = null;
        
      if (elemIds.Count > 0)
      {
        foreach (ElementId elemId in elemIds)
        {
          // ŏ̃AZuvf̃JeS[gp

          keyElemId = elemId;
          categoryId = doc.get_Element(elemId).Category.Id;
          break;
        }

        Transaction transaction = new Transaction(doc);
          
        // JeS[LmF

        if (AssemblyInstance.IsValidNamingCategory(
          doc, categoryId, elemIds
          ))
        {
          transaction.Start("Create Assembly Instance");
          AssemblyInstance assemblyInstance = 
            AssemblyInstance.Create(doc, elemIds, categoryId);

          // CX^XύXOɃAZu[CX^X쐬gUNVR~bg

          transaction.Commit(); 

          // AZu[CX^XɖOATCgUNVJn

          transaction.Start("Set Assembly Name");
          assemblyInstance.AssemblyTypeName = 
            "My Assembly Name " + keyElemId.ToString();
          transaction.Commit();

          // AZu[CX^XɃAZu[r[쐬
                    
          if (assemblyInstance.AllowsAssemblyViewCreation()) 
          {
            transaction.Start("View Creation");

            // RDr[쐬

            View3D view3d = AssemblyViewUtils.Create3DOrthographic(
              doc, assemblyInstance.Id
              );

            // p[gXg쐬

            View partList = AssemblyViewUtils.CreatePartList(
              doc, assemblyInstance.Id
              );
            transaction.Commit();
          }
        }
      }

      return Autodesk.Revit.UI.Result.Succeeded;
    }
    #endregion
  }

  [Transaction(TransactionMode.Manual)]
  public class CompareAssemblies : IExternalCommand
  {
    public Result  Execute(
      ExternalCommandData commandData, 
      ref string message, 
      ElementSet elements
      )
    {

      // OR}hf[^AvP[VƃhLg擾

      UIApplication uiapp = commandData.Application;
      UIDocument uidoc = uiapp.ActiveUIDocument;
      Document doc = uidoc.Document;

      // Q̃AZu[̃CX^XI鎖𑣂

      IList<Reference> assemblyRefs = uidoc.Selection.PickObjects(
        Autodesk.Revit.UI.Selection.ObjectType.Element,
        "AZu[QI" // Pick Two Assemblies
        );
      FindAssemblyDifferences(doc.GetElement(
        assemblyRefs[0]) as AssemblyInstance, 
        doc.GetElement(assemblyRefs[1]) as AssemblyInstance
        );

      return Autodesk.Revit.UI.Result.Succeeded;
    }


    private void FindAssemblyDifferences(
      AssemblyInstance firstInstance, 
      AssemblyInstance secondInstance
      )
    {
      Autodesk.Revit.DB.Document doc = firstInstance.Document;

      // AZu[CX^Xr

      AssemblyDifference difference = 
        AssemblyInstance.CompareAssemblyInstances(
        firstInstance, 
        secondInstance
        );

      // Ⴂ͖mF

      AssemblyDifferenceNone diffNone = 
        difference as AssemblyDifferenceNone;
      if (diffNone != null)
      {
          TaskDialog.Show("Revit", "AZu[͈v܂"); // Assemblies are identical
        return;
      }

      // ݒ肪قȂ鎖mF

      AssemblyDifferenceConfiguration diffConfig = 
        difference as AssemblyDifferenceConfiguration;
      if (diffConfig != null)
      {
        TaskDialog.Show(
          "Revit",
          "vf̔z񂪈قȂ" // Elements are not arranged in space relative to each other in the same way
          );
        return;
      }

      // o[قȂ鎖mF

      AssemblyDifferenceMemberCount diffMemberCount = 
        difference as AssemblyDifferenceMemberCount;
      if (diffMemberCount != null)
      {
        TaskDialog.Show(
          "Revit",
          "vf̈Ⴂ: " + diffMemberCount.Count1 + // Elements counts differ: 
          " & " + diffMemberCount.Count2
          );
        return;
      }

      // JeS[قȂ鎖mF 

      AssemblyDifferenceNamingCategory diffNameCat = 
        difference as AssemblyDifferenceNamingCategory;
      if (diffNameCat != null)
      {
        int catInt1 = diffNameCat.NamingCategoryId1.IntegerValue;
        int catInt2 = diffNameCat.NamingCategoryId2.IntegerValue;
        TaskDialog.Show("Revit",
          "JeS[̈Ⴂ: " + // Naming categories differ: 
          doc.Settings.Categories.get_Item(
          (BuiltInCategory)catInt1).Name + " & " +
          doc.Settings.Categories.get_Item(
          (BuiltInCategory)catInt2).Name);
        return;
      }

      AssemblyDifferenceMemberDifference memberDifference = 
        difference as AssemblyDifferenceMemberDifference;
      if (memberDifference != null)
      {
        AssemblyMemberDifference assemblyMemberDifference = 
          memberDifference.MemberDifference 
          as AssemblyMemberDifference;

        AssemblyMemberDifferentCategory differentCategory = 
          assemblyMemberDifference 
          as AssemblyMemberDifferentCategory;
        if (differentCategory != null)
        {
          int int1 = differentCategory.CategoryId1.IntegerValue;
          int int2 = differentCategory.CategoryId2.IntegerValue;
          TaskDialog.Show("Revit",
            "vfJeS[̈Ⴂ: " + // Element categories differ: 
            doc.Settings.Categories.get_Item(
            (BuiltInCategory)int1).Name + " & " +
            doc.Settings.Categories.get_Item(
            (BuiltInCategory)int2).Name);
          return;
        }
        
        AssemblyMemberDifferentGeometry differentGeometry = 
          assemblyMemberDifference 
          as AssemblyMemberDifferentGeometry;
        if (differentGeometry != null)
        {
          TaskDialog.Show("Revit", "o[WIg[̈Ⴂ"); // Member geometry differs
          return;
        }
        AssemblyMemberDifferentParameters differentParameters = 
          assemblyMemberDifference 
          as AssemblyMemberDifferentParameters;
        if (differentParameters != null)
        {
            TaskDialog.Show("Revit", "o[p[^̈Ⴂ"); // Member parameters differs
          return;
        }
        AssemblyMemberDifferentType differentType = 
          assemblyMemberDifference as AssemblyMemberDifferentType;
        if (differentType != null)
        {
          TaskDialog.Show("Revit",
            "vf^Cv̈Ⴂ: " + //Element types differ: 
            doc.get_Element(differentType.TypeId1).Name + 
            " & " + 
            doc.get_Element(differentType.TypeId2).Name);
          return;
        }
      }
      return;
    }
  }
}